package edu.umich.visualsoar.ruleeditor;

import java.io.*;
import java.util.TreeMap;
import java.awt.*;
import javax.swing.text.*;
import java.util.Enumeration;
import edu.umich.visualsoar.parser.*;
import edu.umich.visualsoar.MainFrame;
import edu.umich.visualsoar.misc.*;

public class SoarDocument extends DefaultStyledDocument {

	AbstractElement 		root = (AbstractElement)getDefaultRootElement();

	SyntaxColor[] 			colorTable;
	
	Preferences				prefs = Preferences.getInstance();
	
	public SoarDocument() {
		colorTable = (SyntaxColor[])Preferences.getInstance().getSyntaxColors().clone();
	}
	
					
	public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
		super.insertString(offset, str, a);
		
		if (! prefs.isHighlightingEnabled()) {
			return;
		}
		
		int		length = str.length();
		
		if (length == 1) {
/*			switch (str.charAt(0)) {
				case ' ':
				case '\n':
				case ')':
				case '>':
				case '.':
				case '\"':
*/					colorSyntax(offset);
			//}	
		}
		else {
			Element	currElem = root.getElement(root.getElementIndex(offset));
			Content data = getContent();
			String elStr = getElementString(currElem,data);
			if(elStr.length() > str.length())
				colorSyntax(currElem.getStartOffset(),elStr.length(),new StringReader(elStr));
			else
				colorSyntax(offset, str.length(), new StringReader(str));
		}	

	}
	
	public void remove(int offs, int len) throws BadLocationException {
		super.remove(offs, len);

		colorSyntax(offs);
	}
            
	
	void replaceRange(String str, int start, int end) {
	    try {
	        remove(start, end - start);
	        insertString(start, str, null);
	    } catch (BadLocationException e) {
	        throw new IllegalArgumentException(e.getMessage());
	    }
	} // replaceRange()
		
	String getElementString(int elementIndex, Content content) {
		String 				theLine = null;
		
		AbstractElement		element;
		
		if (elementIndex > -1) {
			element = (AbstractElement)root.getElement(elementIndex);
		}
		else {
			return null;
		}

		try {
			theLine = content.getString(element.getStartOffset(),
										element.getEndOffset() - element.getStartOffset() - 1);
        } catch (BadLocationException e) {
			e.printStackTrace();
		}
		
		return theLine;
	} // getElementString()

	String getElementString(Element element, Content content) {
		String 				theLine = null;
		
		try {
			theLine = content.getString(element.getStartOffset(),
										element.getEndOffset() - element.getStartOffset() - 1);
        } catch (BadLocationException e) {
			e.printStackTrace();
		}
		
		return theLine;
	} // getElementString()
	
	String getElementString(Element element, Content content, int startOffset) {
		String 	theLine = null;
		
		try {
			theLine = content.getString(startOffset,
										element.getEndOffset() - startOffset - 1);
        } catch (BadLocationException e) {
			e.printStackTrace();
		}
		
		return theLine;
	} // getElementString()

	void colorRange(int begPos, int length, int kind) {
		Color 				theColor = colorTable[kind];
		SimpleAttributeSet 	attrib = new SimpleAttributeSet();
		String text = null;
		try {
			text = getText(begPos,length);
		}
		catch(BadLocationException ble) {}
		
		StyleConstants.setForeground(attrib, theColor);				
		setCharacterAttributes(begPos, length, attrib, false);
	}

	void evaluateToken(Token currToken, int startOffset,SoarParserTokenManager mgr) {
		Token	nextToken;

		int		begin;
		int		length;
	
	    Token startToken = currToken;
		switch (currToken.kind) {
			case SoarParserConstants.RARROW :
				begin = startOffset + currToken.beginColumn;
				length = 3;
				colorRange(begin, length, currToken.kind);
				break;
				
			case SoarParserConstants.SP :
				begin = startOffset + currToken.beginColumn;
				length = 2;
				colorRange(begin, length, currToken.kind);
				break;
									
			case SoarParserConstants.CARET : // followed by a STRING
				begin = 1+ startOffset + currToken.beginColumn;
				
				currToken = mgr.getNextToken();
				if ((currToken.kind == SoarParserConstants.SYMBOLIC_CONST)
				 || (currToken.kind == SoarParserConstants.INTEGER_CONST)
				 || (currToken.kind == SoarParserConstants.FLOATING_POINT_CONST)) {
				 
					length = currToken.image.length();
					colorRange(begin, length, SoarParserConstants.CARET);					 
					
				 	currToken = mgr.getNextToken();
				 	while (currToken.kind == SoarParserConstants.PERIOD) {
				 		begin += length + 1; // don't color period
				 	
				 		currToken = mgr.getNextToken();
				 		length = currToken.image.length();
						
						if ((currToken.kind == SoarParserConstants.SYMBOLIC_CONST)
						 || (currToken.kind == SoarParserConstants.INTEGER_CONST)
						 || (currToken.kind == SoarParserConstants.FLOATING_POINT_CONST)) {

							colorRange(begin, length, SoarParserConstants.CARET);
						}
						else if (currToken.kind == SoarParserConstants.VARIABLE) {
							colorRange(begin, length, SoarParserConstants.VARIABLE);
						}
						
						currToken = mgr.getNextToken();
				 	}
				 					
				}
				else if (currToken.kind == SoarParserConstants.VARIABLE) {
				 
					length = currToken.image.length();
					colorRange(begin, length, SoarParserConstants.VARIABLE);					 
					
				 	currToken = mgr.getNextToken();
				 	while (currToken.kind == SoarParserConstants.PERIOD) {
				 		begin += length + 1; // don't color period
				 	
				 		currToken = mgr.getNextToken();
				 		length = currToken.image.length();
						
						if ((currToken.kind == SoarParserConstants.SYMBOLIC_CONST)
						 || (currToken.kind == SoarParserConstants.INTEGER_CONST)
						 || (currToken.kind == SoarParserConstants.FLOATING_POINT_CONST)) {

							colorRange(begin, length, SoarParserConstants.CARET);
						}
						else if (currToken.kind == SoarParserConstants.VARIABLE) {
							colorRange(begin, length, SoarParserConstants.VARIABLE);
						}
						
						currToken = mgr.getNextToken();
				 	}
				 					
				}
				// We need to make sure that this token is on the same line as the first
				// token
				if(startToken.beginLine != currToken.beginLine) {
					Element rootElement = getDefaultRootElement();
					Element childElement = rootElement.getElement(currToken.beginLine);
					evaluateToken(currToken,childElement.getStartOffset()-1,mgr);
				}
				else
					evaluateToken(currToken, startOffset, mgr);  
				break;
				
			case SoarParserConstants.VARIABLE :
				begin = startOffset + currToken.beginColumn;
				// XXX Assumes that tokens do not cross line barriers
				length = currToken.image.length();
				colorRange(begin, length, currToken.kind);
				break;
				
			case SoarParserConstants.SYMBOLIC_CONST :
				begin = startOffset + currToken.beginColumn;
				// XXX Assumes that tokens do not cross line barriers
				length = currToken.image.length();
				colorRange(begin, length, currToken.kind);
				break;
			default:
				break;
				
		} // token cases
		
	} // evaluateToken()
	
	public void colorSyntax(Reader r) {
		(new ColorSyntaxThread(r)).start();
	}

	class ColorSyntaxThread extends Thread {
		Reader	r;
	
		public ColorSyntaxThread(Reader inReader) {
			r = inReader;
		}	
		
		public void run() {
			colorSyntax();
		}

		public void colorSyntax() {
			SoarParserTokenManager 	mgr = new SoarParserTokenManager(
										  	new ASCII_CharStream(r,0,0));
										 	
			Token 					currToken;
			
			Element					currElem;
			
			int						currLineNum;
			int						offset;
					
			try {
				currToken = mgr.getNextToken();
			} catch (TokenMgrError tme) {
				/* this just means the syntax wasn't valid at
				 * the current state of entry we assume that more
				 * is coming and give up for now
				 */
				 return;
			}
			
			currLineNum = currToken.beginLine;
			currElem = root.getElement(currLineNum);		
							
			while (currToken.kind != SoarParserConstants.EOF) {
			
				if (currLineNum != currToken.beginLine) {
					currLineNum = currToken.beginLine;
					currElem = root.getElement(currLineNum);		
				}
				
				if (currLineNum == 0) {
					offset = currElem.getStartOffset();
				}
				else {
					offset = currElem.getStartOffset() - 1;
				}
				
				evaluateToken(currToken, offset, mgr);
				
				try {
					currToken = mgr.getNextToken();
				} catch (TokenMgrError tme) {
					// see init of currToken...
					 return;
				}
				
			}  // iterate through tokens	
		} // colorSyntax() (whole file)
		
	} // ColorSyntaxThread

	public void colorSyntax(int caretPos) {	
		Content		 			data = getContent();

		SoarParserTokenManager 	mgr;
	
		Token					currToken;
				
		Element					currElem = root.getElement(root.getElementIndex(caretPos));
		
		String					currLine;
		
		int 					offset = currElem.getStartOffset();
		
		currLine = getElementString(currElem, data);	
		
		mgr = new SoarParserTokenManager(new ASCII_CharStream(new StringReader(currLine), 0, 0));
		
		try {
			currToken = mgr.getNextToken();
		} catch (TokenMgrError tme) {
			/* this just means the syntax wasn't valid at
			 * the current state of entry we assume that more
			 * is coming and give up for now
			 */
			 return;
		}
		
		// init all the text to black
		colorRange(offset, currLine.length(), SoarParserConstants.DEFAULT);
				
		while (currToken.kind != SoarParserConstants.EOF) {
		
			evaluateToken(currToken, offset, mgr);
			
			try {
				currToken = mgr.getNextToken();
			} catch (TokenMgrError tme) {
				// see init of currToken...
				 return;
			}
			
		}  // iterate through tokens	

		
	} // colorSyntax (one line)
	
	
	public void colorSyntax(int caretPos, int length, Reader r) {
		Content		data = getContent();

		String		modifiedString;
		
		Element		currElem;
		
		int			startLineNum = root.getElementIndex(caretPos);
		int			currLineNum;
		int			startPos;
		int			endPos;
		int 		totalLength; 
		
		currElem = root.getElement(startLineNum);
		startPos = currElem.getStartOffset();
		endPos = root.getElement(root.getElementIndex(caretPos + length)).getEndOffset() - 1;
		totalLength = endPos - startPos; 

		SoarParserTokenManager 	mgr = new SoarParserTokenManager(
									   new ASCII_CharStream(r, 0, 0));
									 	
		Token 					currToken;
		
				
		try {
			currToken = mgr.getNextToken();
		} catch (TokenMgrError tme) {
			/* this just means the syntax wasn't valid at
			 * the current state of entry we assume that more
			 * is coming and give up for now
			 */
			 return;
		}
		
		currLineNum = startLineNum + currToken.beginLine;
		
		// init all the text to black
		//colorRange(startPos, totalLength, SoarParserConstants.DEFAULT);
		
		while (currToken.kind != SoarParserConstants.EOF) {
		
			currLineNum = startLineNum + currToken.beginLine;
			currElem = root.getElement(currLineNum);
			
			if (currToken.beginLine == 0) {
				evaluateToken(currToken, currElem.getStartOffset(), mgr);
			}
			else {
				evaluateToken(currToken, currElem.getStartOffset() - 1, mgr);
			}
			
			try {
				currToken = mgr.getNextToken();
			} catch (TokenMgrError tme) {
				// see init of currToken...
				 return;
			}
			
		}  // iterate through tokens	
	} // colorSyntax() (specific section)

	public int autoJustify(int caretPos) {
		if(!edu.umich.visualsoar.misc.Preferences.getInstance().isAutoIndentingEnabled())
			return -1;
		Content 			data = getContent();
		
		String				prevLine = null;
		String				currLine = null;
		String				afterCaretString = null;
		String				newCurrLine = null;
		String				indentString = "";
		
		
		int 				elemIndex = root.getElementIndex(caretPos);
		int					numSpaces = 0;
		int					currLineBegin;
		int					currLineEnd;
		
		
		char				lastChar;
		char[]				indentChars;
		
		boolean				leftOfText = true;
		
		AbstractElement 	currLineElem;


		currLineElem = (AbstractElement)root.getElement(elemIndex);
		currLineBegin = currLineElem.getStartOffset();
		currLineEnd = currLineElem.getEndOffset() - 1;
		currLine = getElementString(currLineElem, data);
		afterCaretString = getElementString(currLineElem, data, caretPos);
		
		if (elemIndex > 0) {
			prevLine = getElementString(elemIndex - 1, data);
		}
		else { // no indent for 1st line
			return -1;
		}
		if (prevLine == null) {
			return -1;
		}
		
		newCurrLine = currLine.trim();
		
		if ((newCurrLine.length() != 0) &&
			(newCurrLine.charAt(0) == '}') || (newCurrLine.startsWith("-->"))) {
				
			numSpaces = 0;
		}			
		// already returned if prevLine == null
		else if (prevLine.trim().length() != 0) {
			lastChar = prevLine.charAt(prevLine.length() - 1);
	        if(prevLine.startsWith("#")) {
				numSpaces = 0;
			}
			else if(prevLine.startsWith("sp")) {
				numSpaces = 3;
			}
		        else if(prevLine.endsWith("-->")) {
			    numSpaces = 3;
		        }
			else if(newCurrLine.startsWith("^") || newCurrLine.startsWith("-^")) {
				String currentLine = prevLine;
			 	int currentElementIndex = elemIndex;
				boolean done = false;	
				numSpaces = 0;
				while(!done && currentLine != null) {
					int upPos = currentLine.indexOf('^');
					if(upPos != -1) {
						numSpaces = upPos;
						done = true;
					}
					else {
						--currentElementIndex;
						currentLine = getElementString(currentElementIndex,data);
					}
				}
			}
			else if (lastChar == ')') {
			    int currentElementIndex = elemIndex - 1;
				boolean done = false;
				numSpaces = 0;
				int count = 0;
				String currentLine = prevLine;
				
				while(!done && currentLine != null) {
					for(int i = currentLine.length() - 1; i >=0; --i) {
						if(currentLine.charAt(i) == ')') {
							++count;
						}
						else if(currentLine.charAt(i) == '(') {
							--count;
							if(count <= 0) {
								numSpaces = i;
								done = true;
								break;
							}
						}
					}
					--currentElementIndex;
					currentLine = getElementString(currentElementIndex,data);
					
				}
			}
			else if (lastChar == '{') {
				numSpaces = prevLine.indexOf('{');
			}
			else if (lastChar == '}') {
				numSpaces = 0;
			}
			else {
				int spacePos = prevLine.lastIndexOf(' ');
				numSpaces = spacePos + 1;
				//numSpaces = prevLine.indexOf('^');
			}				
		}
		// else { numSpaces = 0; } , already initialized				
		
		// properly line up negated conditions
		if ((newCurrLine.length() != 0) && (newCurrLine.charAt(0) == '-') &&
			(numSpaces > 0)) {
			
			numSpaces--;
		} 
		
		// no line up char found, or coincidental 3 space indent
		if ((numSpaces == 3) || (numSpaces < 0)) {
			indentString = "   ";
			numSpaces = 3;
		}
		// variable indent to line up chars vertically
		else if (numSpaces > 0) { 
			indentChars = new char[numSpaces];
			
			for (int i = 0; i < numSpaces; i++) {
				indentChars[i] = ' ';
			}
			
			indentString = new String(indentChars);
		}
		// no indent for normal chars, or coincidental 0 space indent
		
		newCurrLine = indentString + newCurrLine;
		
		if (! newCurrLine.equals(currLine)) { // not already justified
			replaceRange(newCurrLine, currLineBegin, currLineEnd);
		}	
		
		/* find out if if the caret is left of all the text on the line	
		 * check for non-whitespace left of the caret
		 */
		for (int i = caretPos - currLineBegin - 1; (i >= 0) && leftOfText; i--) {
			if (! Character.isWhitespace(currLine.charAt(i))) {
				leftOfText = false;
			}
		}
	
		if (leftOfText) {
			return (currLineBegin + numSpaces);
		}
		else { // caret is in the middle, or after text
			return (newCurrLine.lastIndexOf(afterCaretString)
											+ currLineBegin);
		}
		
	} // autoJustify()

} // class SoarDocument
